@@ -9,12 +9,14 @@
 #include "accountfwd.h"
 #include "capabilities.h"
 #include "clientsideencryptionjobs.h"
+#include "common/utility.h"
 #include "configfile.h"
 #include "cookiejar.h"
 #include "creds/abstractcredentials.h"
 #include "networkjobs.h"
 #include "pushnotifications.h"
 #include "theme.h"
+#include "updatechannel.h"
 #include "version.h"
 
 #include "deletejob.h"
@@ -84,6 +86,7 @@ AccountPtr Account::create()
 {
     AccountPtr acc = AccountPtr(new Account);
     acc->setSharedThis(acc);
+    acc->_e2e.setAccount(acc);
     return acc;
 }
 
@@ -192,19 +195,18 @@ QString Account::shortcutName() const
 
     auto shortcutName = QStringLiteral("%1 - %2").arg(url_part, prettyName());
 
-
     return shortcutName;
 }
 
 QString Account::userIdAtHostWithPort() const
 {
-    QString dn = QStringLiteral("%1@%2").arg(_davUser, _url.host());
+    auto host = QStringLiteral("%1@%2").arg(_davUser, _url.host());
     const auto port = url().port();
     if (port > 0 && port != 80 && port != 443) {
-        dn.append(QLatin1Char(':'));
-        dn.append(QString::number(port));
+        host.append(QLatin1Char(':'));
+        host.append(QString::number(port));
     }
-    return dn;
+    return host;
 }
 
 QString Account::davDisplayName() const
@@ -522,7 +524,7 @@ QSslConfiguration Account::getOrCreateSslConfig()
 
     // if setting the client certificate fails, you will probably get an error similar to this:
     //  "An internal error number 1060 happened. SSL handshake failed, client certificate was requested: SSL error: sslv3 alert handshake failure"
-    QSslConfiguration sslConfig = QSslConfiguration::defaultConfiguration();
+    auto sslConfig = QSslConfiguration::defaultConfiguration();
 
     // Try hard to reuse session for different requests
     sslConfig.setSslOption(QSsl::SslOptionDisableSessionTickets, false);
@@ -557,7 +559,7 @@ void Account::setSslErrorHandler(AbstractSslErrorHandler *handler)
 
 void Account::setUrl(const QUrl &url)
 {
-    const QRegularExpression discoverPublicLinks(R"(((https|http)://[^/]*).*/s/([^/]*))");
+    const QRegularExpression discoverPublicLinks(R"(((https|http)://[^/]*).*/s/([^/]*)$)");
     const auto isPublicLink = discoverPublicLinks.match(url.toString());
     if (isPublicLink.hasMatch()) {
         _url = QUrl::fromUserInput(isPublicLink.captured(1));
@@ -599,7 +601,7 @@ QVariant Account::credentialSetting(const QString &key) const
 void Account::setCredentialSetting(const QString &key, const QVariant &value)
 {
     if (_credentials) {
-        QString prefix = _credentials->authType();
+        const auto prefix = _credentials->authType();
         _settingsMap.insert(prefix + "_" + key, value);
     }
 }
@@ -645,8 +647,9 @@ void Account::slotHandleSslErrors(QNetworkReply *reply, QList<QSslError> errors)
     QPointer<QObject> guard = reply;
 
     if (_sslErrorHandler->handleErrors(errors, reply->sslConfiguration(), &approvedCerts, sharedFromThis())) {
-        if (!guard)
+        if (!guard) {
             return;
+        }
 
         if (!approvedCerts.isEmpty()) {
             QSslConfiguration::defaultConfiguration().addCaCertificates(approvedCerts);
@@ -662,8 +665,9 @@ void Account::slotHandleSslErrors(QNetworkReply *reply, QList<QSslError> errors)
         // certificate changes.
         reply->ignoreSslErrors(errors);
     } else {
-        if (!guard)
+        if (!guard) {
             return;
+        }
 
         // Mark all involved certificates as rejected, so we don't ask the user again.
         for (const auto &error : std::as_const(errors)) {
@@ -696,10 +700,11 @@ void Account::slotCredentialsFetched()
             emit credentialsFetched(_credentials.data());
         });
         fetchUserNameJob->start();
-    } else {
-        qCDebug(lcAccount) << "User id already fetched.";
-        emit credentialsFetched(_credentials.data());
+        return;
     }
+
+    qCDebug(lcAccount) << "User id already fetched.";
+    emit credentialsFetched(_credentials.data());
 }
 
 void Account::slotCredentialsAsked()
@@ -785,7 +790,7 @@ void Account::resetShouldSkipE2eeMetadataChecksumValidation()
 int Account::serverVersionInt() const
 {
     // FIXME: Use Qt 5.5 QVersionNumber
-    auto components = serverVersion().split('.');
+    const auto components = serverVersion().split('.');
     return makeServerVersion(components.value(0).toInt(),
         components.value(1).toInt(),
         components.value(2).toInt());
@@ -849,39 +854,43 @@ void Account::setServerVersion(const QString &version)
         return;
     }
 
-    auto oldServerVersion = _serverVersion;
+    const auto oldServerVersion = _serverVersion;
     _serverVersion = version;
     emit serverVersionChanged(sharedFromThis(), oldServerVersion, version);
 }
 
-void Account::writeAppPasswordOnce(QString appPassword){
-    if(_wroteAppPassword)
+void Account::writeAppPasswordOnce(const QString &appPassword)
+{
+    if (_wroteAppPassword) { 
         return;
+    }
 
     // Fix: Password got written from Account Wizard, before finish.
     // Only write the app password for a connected account, else
     // there'll be a zombie keychain slot forever, never used again ;p
     //
     // Also don't write empty passwords (Log out -> Relaunch)
-    if(id().isEmpty() || appPassword.isEmpty())
+    if (id().isEmpty() || appPassword.isEmpty()) {
         return;
+    }
 
-    const QString kck = AbstractCredentials::keychainKey(
+    const auto keychainKey = AbstractCredentials::keychainKey(
                 url().toString(),
                 davUser() + app_password,
                 id()
     );
 
-    auto *job = new WritePasswordJob(Theme::instance()->appName());
+    auto job = new WritePasswordJob(Theme::instance()->appName());
     job->setInsecureFallback(false);
-    job->setKey(kck);
+    job->setKey(keychainKey);
     job->setBinaryData(appPassword.toLatin1());
     connect(job, &WritePasswordJob::finished, [this](Job *incoming) {
-        auto *writeJob = dynamic_cast<WritePasswordJob *>(incoming);
-        if (writeJob->error() == NoError)
+        auto writeJob = dynamic_cast<WritePasswordJob *>(incoming);
+        if (writeJob->error() == NoError) {
             qCInfo(lcAccount) << "appPassword stored in keychain";
-        else
+        } else {
             qCWarning(lcAccount) << "Unable to store appPassword in keychain" << writeJob->errorString();
+        }
 
         // We don't try this again on error, to not raise CPU consumption
         _wroteAppPassword = true;
@@ -889,49 +898,49 @@ void Account::writeAppPasswordOnce(QString appPassword){
     job->start();
 }
 
-void Account::retrieveAppPassword(){
-    const QString key = credentials()->user() + app_password;
-    const QString kck = AbstractCredentials::keychainKey(
+void Account::retrieveAppPassword()
+{
+    const auto key = QString(credentials()->user() + app_password);
+    const auto keychainKey = AbstractCredentials::keychainKey(
                 url().toString(),
                 key,
                 id()
     );
 
-    auto *job = new ReadPasswordJob(Theme::instance()->appName());
+    auto job = new ReadPasswordJob(Theme::instance()->appName());
     job->setInsecureFallback(false);
-    job->setKey(kck);
+    job->setKey(keychainKey);
     connect(job, &ReadPasswordJob::finished, [this](Job *incoming) {
-        auto *readJob = dynamic_cast<ReadPasswordJob *>(incoming);
-        QString pwd("");
+        auto readJob = dynamic_cast<ReadPasswordJob *>(incoming);
+        const auto password = readJob->binaryData();
         // Error or no valid public key error out
-        if (readJob->error() == NoError &&
-                readJob->binaryData().length() > 0) {
-            pwd = readJob->binaryData();
+        if (readJob->error() == NoError && password.length() > 0) {
+            qCDebug(lcAccount) << "Found appPassword";
         }
 
-        emit appPasswordRetrieved(pwd);
+        emit appPasswordRetrieved(password);
     });
     job->start();
 }
 
 void Account::deleteAppPassword()
 {
-    const QString kck = AbstractCredentials::keychainKey(
+    const auto keychainKey = AbstractCredentials::keychainKey(
                 url().toString(),
                 credentials()->user() + app_password,
                 id()
     );
 
-    if (kck.isEmpty()) {
+    if (keychainKey.isEmpty()) {
         qCDebug(lcAccount) << "appPassword is empty";
         return;
     }
 
-    auto *job = new DeletePasswordJob(Theme::instance()->appName());
+    auto job = new DeletePasswordJob(Theme::instance()->appName());
     job->setInsecureFallback(false);
-    job->setKey(kck);
+    job->setKey(keychainKey);
     connect(job, &DeletePasswordJob::finished, [this](Job *incoming) {
-        auto *deleteJob = dynamic_cast<DeletePasswordJob *>(incoming);
+        auto deleteJob = dynamic_cast<DeletePasswordJob *>(incoming);
         const auto jobError = deleteJob->error();
         if (jobError == NoError) {
             qCInfo(lcAccount) << "appPassword deleted from keychain";
@@ -968,16 +977,17 @@ void Account::deleteAppToken()
 
 void Account::fetchDirectEditors(const QUrl &directEditingURL, const QString &directEditingETag)
 {
-    if(directEditingURL.isEmpty() || directEditingETag.isEmpty())
+    if (directEditingURL.isEmpty() || directEditingETag.isEmpty()) {
         return;
+    }
 
     // Check for the directEditing capability
     if (!directEditingURL.isEmpty() &&
         (directEditingETag.isEmpty() || directEditingETag != _lastDirectEditingETag)) {
-            // Fetch the available editors and their mime types
-            auto *job = new JsonApiJob(sharedFromThis(), QLatin1String("ocs/v2.php/apps/files/api/v1/directEditing"));
-            QObject::connect(job, &JsonApiJob::jsonReceived, this, &Account::slotDirectEditingRecieved);
-            job->start();
+        // Fetch the available editors and their mime types
+        auto job = new JsonApiJob(sharedFromThis(), QLatin1String("ocs/v2.php/apps/files/api/v1/directEditing"));
+        QObject::connect(job, &JsonApiJob::jsonReceived, this, &Account::slotDirectEditingRecieved);
+        job->start();
     }
 }
 
@@ -1041,7 +1051,7 @@ void Account::setLockFileState(const QString &serverRelativePath,
                                const SyncFileItem::LockStatus lockStatus,
                                const SyncFileItem::LockOwnerType lockOwnerType)
 {
-    auto& lockStatusJobInProgress = _lockStatusChangeInprogress[serverRelativePath];
+    auto &lockStatusJobInProgress = _lockStatusChangeInprogress[serverRelativePath];
     if (lockStatusJobInProgress.contains(lockStatus)) {
         qCWarning(lcAccount) << "Already running a job with lockStatus:" << lockStatus << " for: " << serverRelativePath;
         return;
@@ -1174,58 +1184,64 @@ void Account::setAskUserForMnemonic(const bool ask)
     emit askUserForMnemonicChanged();
 }
 
-void Account::listRemoteFolder(QPromise<OCC::PlaceholderCreateInfo> *promise, const QString &path, SyncJournalDb *journalForFolder)
+void Account::listRemoteFolder(QPromise<OCC::PlaceholderCreateInfo> *promise, const QString &remoteSyncRootPath, const QString &subPath, SyncJournalDb *journalForFolder)
 {
-    qCInfo(lcAccount()) << "ls col job requested for" << path;
+    qCInfo(lcAccount()) << "ls col job requested for" << subPath;
 
     if (!credentials()->ready()) {
-        qCWarning(lcAccount()) << "credentials are not ready" << path;
+        qCWarning(lcAccount()) << "credentials are not ready" << subPath;
         promise->finish();
         return;
     }
 
-    auto listFolderJob = new OCC::LsColJob{sharedFromThis(), path};
+    auto listFolderJob = new OCC::LsColJob{sharedFromThis(), subPath};
 
     const auto props = LsColJob::defaultProperties(LsColJob::FolderType::ChildFolder, sharedFromThis());
 
     listFolderJob->setProperties(props);
 
-    QObject::connect(listFolderJob, &OCC::LsColJob::networkError, this, [promise, path] (QNetworkReply *reply) {
+    QObject::connect(listFolderJob, &OCC::LsColJob::networkError, this, [promise, subPath] (QNetworkReply *reply) {
         if (reply) {
-            qCWarning(lcAccount()) << "ls col job" << path << "error" << reply->errorString();
+            qCWarning(lcAccount()) << "ls col job" << subPath << "error" << reply->errorString();
         }
 
-        qCWarning(lcAccount()) << "ls col job" << path << "error without a reply";
+        qCWarning(lcAccount()) << "ls col job" << subPath << "error without a reply";
         promise->finish();
     });
 
-    QObject::connect(listFolderJob, &OCC::LsColJob::finishedWithError, this, [promise, path] (QNetworkReply *reply) {
+    QObject::connect(listFolderJob, &OCC::LsColJob::finishedWithError, this, [promise, subPath] (QNetworkReply *reply) {
         if (reply) {
-            qCWarning(lcAccount()) << "ls col job" << path << "error" << reply->errorString();
+            qCWarning(lcAccount()) << "ls col job" << subPath << "error" << reply->errorString();
         }
 
-        qCWarning(lcAccount()) << "ls col job" << path << "error without a reply";
+        qCWarning(lcAccount()) << "ls col job" << subPath << "error without a reply";
         promise->finish();
     });
 
-    QObject::connect(listFolderJob, &OCC::LsColJob::finishedWithoutError, this, [promise, path] () {
-        qCInfo(lcAccount()) << "ls col job" << path << "finished";
+    QObject::connect(listFolderJob, &OCC::LsColJob::finishedWithoutError, this, [promise, subPath] () {
+        qCInfo(lcAccount()) << "ls col job" << subPath << "finished";
         promise->finish();
     });
 
-    auto ignoreFirst = true;
-    QObject::connect(listFolderJob, &OCC::LsColJob::directoryListingIterated, this, [&ignoreFirst, promise, path, journalForFolder, this] (const QString &name, const QMap<QString, QString> &properties) {
-        if (ignoreFirst) {
+    listFolderJob->setProperty("ignoredFirst", false);
+    const auto baseRemotePath = Utility::trailingSlashPath(Utility::noLeadingSlashPath(remoteSyncRootPath));
+    auto syncRootPath = subPath;
+    if (subPath.startsWith(baseRemotePath)) {
+        syncRootPath = syncRootPath.mid(baseRemotePath.size());
+    }
+
+    QObject::connect(listFolderJob, &OCC::LsColJob::directoryListingIterated, this, [promise, remoteSyncRootPath, syncRootPath, journalForFolder, this](const QString &completeDavPath, const QMap<QString, QString> &properties) {
+        if (!sender()->property("ignoredFirst").toBool()) {
             qCDebug(lcAccount()) << "skip first item";
-            ignoreFirst = false;
+            sender()->setProperty("ignoredFirst", true);
             return;
         }
 
-        qCInfo(lcAccount()) << "ls col job" << path << "new file" << name << properties.count();
+        qCInfo(lcAccount()) << "ls col job" << syncRootPath << "new file" << completeDavPath << properties.count();
 
-        const auto slash = name.lastIndexOf('/');
-        const auto itemFileName = name.mid(slash + 1);
-        const auto absoluteItemPathName = (path.isEmpty() ? itemFileName : path + "/" + itemFileName);
+        const auto slash = completeDavPath.lastIndexOf('/');
+        const auto itemFileName = completeDavPath.mid(slash + 1);
+        const auto absoluteItemPathName = syncRootPath.isEmpty() ? itemFileName : Utility::noTrailingSlashPath(syncRootPath) + '/' + itemFileName;
 
         auto currentItemDbRecord = SyncJournalFileRecord{};
         if (journalForFolder->getFileRecord(absoluteItemPathName, &currentItemDbRecord) && currentItemDbRecord.isValid()) {
@@ -1257,6 +1273,16 @@ void Account::setServerHasValidSubscription(bool valid)
     _serverHasValidSubscription = valid;
 }
 
+UpdateChannel Account::enterpriseUpdateChannel() const
+{
+    return _enterpriseUpdateChannel;
+}
+
+void Account::setEnterpriseUpdateChannel(const UpdateChannel &channel)
+{
+    _enterpriseUpdateChannel = channel;
+}
+
 void Account::updateServerSubcription()
 {
     ConfigFile currentConfig;
@@ -1271,10 +1297,13 @@ void Account::updateServerSubcription()
 
 void Account::updateDesktopEnterpriseChannel()
 {
+    const auto capabilityEnterpriseChannel = UpdateChannel::fromString(_capabilities.desktopEnterpriseChannel());
+    _enterpriseUpdateChannel = capabilityEnterpriseChannel;
+
     ConfigFile currentConfig;
-    if (const auto desktopEnterpriseChannel = _capabilities.desktopEnterpriseChannel();
-        desktopEnterpriseChannel != currentConfig.desktopEnterpriseChannel()) {
-        currentConfig.setDesktopEnterpriseChannel(desktopEnterpriseChannel);
+    const auto configEnterpriseChannel = UpdateChannel::fromString(currentConfig.desktopEnterpriseChannel());
+    if (capabilityEnterpriseChannel > configEnterpriseChannel) {
+        currentConfig.setDesktopEnterpriseChannel(capabilityEnterpriseChannel.toString());
     }
 }
 
@@ -1423,7 +1452,14 @@ void Account::setUploadLimitSetting(const AccountNetworkTransferLimitSetting set
         return;
     }
 
-    _uploadLimitSetting = setting;
+    auto targetSetting = setting;
+
+    if (setting == AccountNetworkTransferLimitSetting::LegacyGlobalLimit) {
+        qCInfo(lcAccount) << "Upload limit setting was requested to be set to the legacy global limit, falling back to unlimited";
+        targetSetting = AccountNetworkTransferLimitSetting::NoLimit;
+    }
+
+    _uploadLimitSetting = targetSetting;
     emit uploadLimitSettingChanged();
 }
 
@@ -1437,8 +1473,15 @@ void Account::setDownloadLimitSetting(const AccountNetworkTransferLimitSetting s
     if (setting == _downloadLimitSetting) {
         return;
     }
-    
-    _downloadLimitSetting = setting;
+
+    auto targetSetting = setting;
+
+    if (setting == AccountNetworkTransferLimitSetting::LegacyGlobalLimit) {
+        qCInfo(lcAccount) << "Download limit setting was requested to be set to the legacy global limit, falling back to unlimited";
+        targetSetting = AccountNetworkTransferLimitSetting::NoLimit;
+    }
+
+    _downloadLimitSetting = targetSetting;
     emit downloadLimitSettingChanged();
 }
 
